/****************************************************************************/
/* LONGFILE.C : Version 1.20, created  1.6.1999, by Leo Eibler (eibler_leo@ */
/* LONGFILE.C : Include File for MP3TAG2.CPP - Long File Names       gmx.at)*/
/****************************************************************************/
/* Version History:                                                         */
/*   1. 6.1999     V1.00 (creating)                                         */
/*   7. 8.1999     Last Update                                              */
/*--------------------------------------------------------------------------*/
/*   1. 6.1999     V1.00: routines for findfirst and findnext               */
/*   8. 6.1999       main test program - nothing works :-(                  */
/*   7. 8.1999     V1.10: routine for stop search and free handle           */
/*                   the main program works !!                              */
/*                 V1.20: if function not supported, normal findfirst will  */
/*                   be called - NOT TESTED                                 */
/****************************************************************************/

/****************************************************************************/
/* Note:                                                                    */
/*  This version is not completely tested and can only list long filenames  */
/*  no timestamps and no accesstimes implemented !!                         */
/****************************************************************************/

/*--------------------------------------------------------------------------*/
/*!                                                                        !*/
/*!  Thanks to Ralph Browns Interrupt List !!                              !*/
/*!                                                                        !*/
/*--------------------------------------------------------------------------*/


//#define TEST_LONGFILE

#ifdef TEST_LONGFILE
  #include <stdio.h>
  #include <conio.h>
  #include <string.h>
  #include <dos.h>
  #include <alloc.h>
  #include <dir.h>
#endif


struct LONGFILE
{
       unsigned long attributes;
       unsigned long create_time1, create_time2;
       unsigned long lastaccess_time1, lastaccess_time2;
       unsigned long lastmodify_time1, lastmodify_time2;
       unsigned long filesize_h, filesize_l;
       char reserved[8];
       char name[260];
       char shortname[14];
} longfile;
char *buf;

struct ffblk ffblk;  // if long filenames not supported,
int done;            // normal findfirst and -next will used

unsigned int longfile_error;
char no_ifs=0;  // if set 1 -> no long file names support

// checks if function 71xx is available (IFS driver loaded)
// if it is available - function returns 1
// else 0
int  check_for_7100(void)
{
     unsigned int AAx;

     asm mov ax,0x714E;  // Findfirst -> Long Filenames under Win 95
     asm mov dx,0;
     asm mov si,dx;
     asm mov cx,0;
     asm mov ds,dx;  // find-pattern
     asm mov dx,0;  // find-pattern
     asm mov es,dx;  // long filename structure
     asm mov di,dx;  // long filename structure
     asm int 0x21;
     asm mov AAx,ax;
     asm jnc ok;
     goto error;
ok:
     return 1;
error:
     if(AAx==0x7100)
       return 0;
     return 1;
}


unsigned int  long_findfirst(char *pattern, unsigned int attributes)
{
     unsigned int offt,segt;
     unsigned int off_s,seg_s;
     unsigned int handle;
     unsigned int AAx,Cf;

     offt=FP_OFF(pattern);
     segt=FP_SEG(pattern);
     off_s=FP_OFF(&longfile);
     seg_s=FP_SEG(&longfile);
     no_ifs=0;

     asm mov ax,0x714E;  // Findfirst -> Long Filenames under Win 95
     asm mov si,0;
     asm mov cx,attributes;
     asm mov dx,segt;
     asm mov ds,dx;  // find-pattern
     asm mov dx,offt;  // find-pattern
     asm mov es,seg_s;  // long filename structure
     asm mov di,off_s;  // long filename structure
     asm int 0x21;
     asm mov AAx,ax;
     asm jnc ok;
     goto error;
ok:
     handle=AAx;
     longfile_error=0;
     goto end;
error:
     handle=0;
     longfile_error=AAx;
     if(AAx==0x7100)
     {
       no_ifs=1;
       findfirst(pattern, &ffblk, attributes);
       strcpy(longfile.name, ffblk.ff_name);
       strcpy(longfile.shortname, ffblk.ff_name);
       longfile.filesize_h=0;
       longfile.filesize_l=ffblk.ff_fsize;
       longfile.attributes=ffblk.ff_attrib;
     }
end:
     return handle;
}

unsigned int  long_findnext(unsigned int handle)
{
     unsigned int off_s,seg_s;
     unsigned int AAx;

     off_s=FP_OFF(&longfile);
     seg_s=FP_SEG(&longfile);
     no_ifs=0;

     asm mov bx,handle;
     asm mov es,seg_s;  // long filename structure
     asm mov di,off_s;  // long filename structure
     asm mov ax,0x714F;  // Findnext -> Long Filenames under Win 95
     asm mov si,0;
     asm int 0x21;
     asm mov AAx,ax;
     asm jnc ok;
     goto error;
ok:
     longfile_error=0;
     goto end;
error:
     longfile_error=AAx;
     if(AAx==0x7100)
     {
       no_ifs=1;
       done=findnext(&ffblk);
       strcpy(longfile.name, ffblk.ff_name);
       strcpy(longfile.shortname, ffblk.ff_name);
       longfile.filesize_h=0;
       longfile.filesize_l=ffblk.ff_fsize;
       longfile_error=done;
       longfile.attributes=ffblk.ff_attrib;
     }
end:
     return longfile_error;
}

void close_long_find(unsigned int handle)
{
     unsigned int AAx;

     if(no_ifs==1)
     {
       longfile_error=0;
       return;
     }
     asm mov ax,0x71A1;  // close handle -> Long Filenames under Win 95
     asm mov bx,handle;
     asm int 0x21;
     asm mov AAx,ax;
     asm jnc ok;
     goto error;
ok:
     longfile_error=0;
     goto end;
error:
     longfile_error=AAx;
     if(AAx==0x7100)
     {
       no_ifs=1;

     }
end:
}


#ifdef TEST_LONGFILE

void main()
{
     unsigned int i, hnd, c;

     clrscr();

     hnd=long_findfirst("*", 0xFF);
     if(hnd==0)
     {
       printf("Error: long_findfirst (AX=%4Xh)\n", longfile_error);
       return;
     }

     printf("Handle=%Xh\n", hnd);

     i=0;
     while(1)
     {
       printf("%d> %s [%ld]\n", i+1, longfile.name, longfile.filesize_l);

       if(long_findnext(hnd)!=0)
         break;
       if(kbhit())
       {
         getch();
         break;
       }
       i++;
       if(i%5==0)
       {
         c=getch();
         if(c==27)
           break;
       }

     }

     close_long_find(hnd);
     if(longfile_error)
       printf("Error while close_long_find(%Xh): AX=%4Xh\n",
              hnd, longfile_error);
}

#endif
